%* Get a list of objects in a folder in SDD                           *;
%*                                                                    *;
%* @param  sdd_path Required. Path in SDD to get the contents of      *;
%* @param  sas_dsname Optional. Name of the dataset to create         *;
%* @param  sdd_recursive Optional. Get objects recursively            *; 
%* @macrotype SDD                                                     *;
%* @exposure internal                                                 *;

%macro sasdrugdev_getacls(sdd_path=, sas_dsname=work.sddGetACLs, sdd_recursive=0)
  /des='Get a list of the objects in a folder in SDD';

	/*  initialize return code and message in case macro aborts          */ ;
	%global _sddRC_ _sddMsg_;
	%let _sddRC_=%str(-99);
	%let _sddMsg_=%str(The SAS Macro did not execute properly.  Unknown error.);
  
	/* assume no records returned                                        */ ;
	%let _sddRowCount_=0;

	/* testing that the sas_dsname is valid before processing */ ;
	data &sas_dsname;
	run;

	%if (&SYSERR ne %str(0)) %then %return; 

	%LOCAL _tempDS_;
	%let _tempDS_=%str(_tempGetACL_ds_);

	/*  start the datastep to call the java code and create the          */;
	/*  resulting dataset                                                */  ; 
	data &_tempDS_;							
		attrib returnCode           length=8;
		attrib message              length=$200;
		attrib rowCount             length=8;
		attrib fullPath             length=$2048    label="Full Path Name";
		attrib path					length=$2048    label="Folder" ;
		attrib ACEname              length=$200     label="Acl object";
		attrib objectType           length=$20      label="SAS Object Type";
		attrib isContainer          length=8        label="isContainer";
		attrib owner                length=$200     label="Owner";
		attrib aclType              length=$20      label="Acl Type";
		attrib principalName        length=$200     label="Principal Name";
		attrib principalType        length=$200     label="Principal Type";
		attrib grpSrcCtxt           length=$2048    label="Group context source path";
		attrib adminPermission      length=$10      label="Admin Permission";
		attrib readPermission       length=$10      label="Read Permission";
		attrib writePropPerm        length=$10      label="Write Properties Permission";
		attrib writeContentPerm     length=$10      label="Write Content Permission";
		attrib deletePermission     length=$10      label="Delete Permission";

		attrib ptype_ACLOwner       length=$30      label="Temp var - aclOwner";
		attrib ptype_ACLMembers     length=$30      label="Temp var - aclMembers";
		attrib ptype_sort           length=8        label="Sorting value";
 
    	attrib logMsgPrefix	        length=$70;

		logMsgPrefix="NOTE: SAS Drug Development Macro: * ";
		rowCount=0;

		/* map the required java commands                                  */;  
		declare javaobj cmd("com/sas/hls/macro/command/ACLCommand", "%bquote(&sdd_path)") ;
		declare javaobj results("com/sas/hls/macro/results/ACLResult");  

		/*  try to get the list of children in a folder                    */ ;
		cmd.callVoidMethod("getAcls", results, "%bquote(&sdd_recursive)");
		/* did it work                                                     */; 
		results.callIntMethod("getReturnCode", returnCode);
		results.callStringMethod("getReturnMessage", message);
		results.callStringMethod("getPrincipalType_ACLOwner", ptype_ACLOwner);
		results.callStringMethod("getPrincipalType_ACLMembers", ptype_ACLMembers);

		/* read the resulting values into a dataset                        */; 
		if (returnCode ne 0) then do;
			logMsgPrefix="ERROR: SAS Drug Development Macro:";
		end;
		else do;
			results.callIntMethod("getObjectCount", rowCount);
			do row=0 to rowCount - 1;
				results.callStringMethod("getAclType", row, aclType);
				results.callStringMethod("getFullPath", row, fullPath);
				results.callStringMethod("getPath", row, path);
				results.callStringMethod("getName", row, ACEname);
				results.callStringMethod("getOwner", row, owner);
				results.callIntMethod("getIsContainer", row, isContainer);
				results.callStringMethod("getObjectType", row, objectType);
				results.callStringMethod("getReadPermission", row, readPermission);
				results.callStringMethod("getPrincipalType", row, principalType);
				results.callStringMethod("getPrincipalName", row, principalName);
				results.callStringMethod("getGroupContextPath", row, grpSrcCtxt);
				results.callStringMethod("getDeletePermission", row, deletePermission);
				results.callStringMethod("getWriteContentPermission", row, writeContentPerm);
				results.callStringMethod("getWritePropertiesPermission", row, writePropPerm);
				results.callStringMethod("getAdminPermission", row, adminPermission);

				if (trim(principalType) eq trim(ptype_ACLOwner)) then
					ptype_sort=1;
				else if (trim(principalType) eq trim(ptype_ACLMembers)) then
					ptype_sort=2;
				else
					ptype_sort=99;

				output;
			end; *end loop through results;
		end; *end rc=0;

		put logMsgPrefix " SAS Drug Development macro sasdrugdev_getacls";
		put logMsgPrefix " Tried to list the objects in %bquote(&sdd_path)";
		if (rowCount=0) then    
			put logMsgPrefix " No objects were found.";
		else
			put logMsgPrefix rowCount " records written to dataset &sas_dsname";
		put logMsgPrefix " Return Code: " returnCode;
		put logMsgPrefix " _sddMsg_= " message ;
		put;

		/*  set the return code and message macro variables                */;
		call symputx("_sddRC_", returnCode);
		call symputx("_sddMsg_", message);
 	run;

	/*  sort the resulting dataset by fullpath                          */ ;
	proc sort data=&_tempDS_ 
			  out=&sas_dsname (label="Created by sasdrugdev_getacls" 
							   keep=aclType fullPath path ACEname owner principalType principalName grpSrcCtxt objectType isContainer 
                                    adminPermission readPermission writePropPerm writeContentPerm deletePermission );
	by fullPath descending aclType ptype_sort principalName;
	run;

	%if (%sysfunc(exist(&_tempDS_))>0) %then %do;
		proc datasets library=work nolist;
			delete &_tempDS_;
		quit; run;
	%end;
	
	%if (%symlocal(_tempDS_)) %then %do;
		%symdel _tempDS_  /nowarn;
	%end;
	
 
%mend sasdrugdev_getacls ;
