%* Get all the versions of an object in a folder in SDD               *;
%*                                                                    *;
%* @param  sdd_path Required. Path in SDD for the object              *;
%* @param  sas_dsname Optional. Name of the dataset to create         *;
%* @macrotype SDD                                                     *;
%* @exposure internal                                                 *;

%macro sasdrugdev_getversions(sdd_path=, sas_dsname=work.sddGetVersions)
  /des='Get all the versions of an object in a folder in SDD';

  %global _sddRC_ _sddMsg_ ;
  %let _sddRC_=%str(-99);
  %let _sddMsg_=%str(The SAS Macro did not execute properly.  Unknown error.);
  
  data &sas_dsname (label="Created by sasdrugdev_getversions" keep=fullPath path name description isContainer isVersioned 
  							versionLabel isCheckedOut isLocked size contentType objectType createdBy createdOn createdOnSAS lastModifiedBy 
  							lastModifiedOn lastModifiedOnSAS propsLastModifiedBy state stateComment isSigned);							
    attrib returnCode           length=8;
    attrib message              length=$200;
    attrib rowCount             length=8;
    attrib fullPath             length=$2048     label="Full Path Name";
    attrib path			        length=$2048     label="Path name only, no file name" ;
    attrib name                 length=$200     label="File or Container Name";
    attrib description          length=$200     label="Description";
    attrib isContainer          length=8        label="Is object a Container?";
    attrib isVersioned          length=8        label="Is file versioned?";
    attrib versionLabel         length=$32      label="Version number";
    attrib isCheckedOut         length=8        label="Is File checked out?";
    attrib isLocked             length=8        label="Is File locked?";
    attrib size                 length=$32      label="Size in Bytes";
    attrib contentType          length=$40      label="Content or Mime Type";
    attrib objectType           length=$40      label="SAS Object Type";
    attrib createdBy            length=$40      label="Userid that created the object";
    attrib createdOn            length=$40      label="Date the object was created (String)";
    attrib createdOnSAS         length=$40      label="Date the object was created (SAS Format)";
    attrib lastModifiedBy       length=$40      label="Userid that last modified the object";
    attrib lastModifiedOn       length=$40      label="Date the object was last modified (String)";
    attrib lastModifiedOnSAS    length=$40      label="Date the object was last modified (SAS Format)";
    attrib propsLastModifiedBy  length=$40      label="Date the properties for the object were last modified";
    attrib state                length=$200     label="State for Analysis or Project" ;
    attrib stateComment         length=$200     label="Comments associated with the state for Analysis or Project";
    attrib isSigned             length=8        label="Does the file have an electronic signature associated with it?";
  
    /* map the required java commands                                  */  
    declare javaobj cmd("com/sas/hls/macro/command/GetVersionsCommand", "%bquote(&sdd_path)") ;
    declare javaobj results("com/sas/hls/macro/results/GetNodesResult");  
    
    /*  try to get the versions for an object                          */ 
    cmd.callVoidMethod("execute", results);
	/* did it work                                                     */ 
    results.callIntMethod("getReturnCode", returnCode);
    results.callStringMethod("getReturnMessage", message);
    
    /* read the resulting values into a dataset                        */ 
    if (returnCode=0) then do;
      results.callIntMethod("getObjectCount", rowCount);
      do row=0 to rowCount - 1;
    	results.callStringMethod("getFullPath", row, fullPath);
        results.callStringMethod("getPath", row, path);
        results.callStringMethod("getName", row, name);
        results.callStringMethod("getDescription", row, description);
        results.callIntMethod("getIsContainer", row, isContainer);
        results.callIntMethod("getIsVersioned", row, isVersioned);
        results.callStringMethod("getVersionLabel", row, versionLabel);
        results.callIntMethod("getIsCheckedOut", row, isCheckedOut);
        results.callIntMethod("getIsLocked", row, isLocked);
        results.callStringMethod("getSize", row, size);
        results.callStringMethod("getContentType", row, contentType);
        results.callStringMethod("getObjectType", row, objectType);
        results.callStringMethod("getCreatedBy", row, createdBy);
        results.callStringMethod("getCreated", row, createdOn);
        results.callStringMethod("getCreatedSASFormat", row, createdOnSAS);
        results.callStringMethod("getLastModifiedBy", row, lastModifiedBy);
        results.callStringMethod("getLastModified", row, lastModifiedOn);
        results.callStringMethod("getLastModifiedSASFormat", row, lastModifiedOnSAS);
        results.callStringMethod("getPropertiesLastModifiedBy", row, propsLastModifiedBy);
        results.callStringMethod("getState", row, state);
        results.callStringMethod("getStateComment", row, stateComment);
        results.callIntMethod("getIsSigned", row, isSigned);
        output;
      end;
      
     /* put a note in the log                                         */  
     put "NOTE: SAS Drug Development Macro: sasdrugdev_getversions";
     put "NOTE: SAS Drug Development Macro: * returned a list of versions for %bquote(&sdd_path)";
     if (rowCount=0) then    
      	put "NOTE: SAS Drug Development Macro: * No versions were found.";
     put "NOTE: SAS Drug Development Macro: * " rowCount " records written to dataset &sas_dsname";
     end;
     else do;
     	put "ERROR: SAS Drug Development Macro: sasdrugdev_getversions";
      	put "ERROR: SAS Drug Development Macro: * Return Code: " returnCode;
      	put "ERROR: SAS Drug Development Macro: * Tried to list the versions for %bquote(&sdd_path)";
      	put "ERROR: SAS Drug Development Macro: * " message;
     end;
     call symputx("_sddRC_", returnCode);
     call symputx("_sddMsg_", message);
     call symputx("_sddRowCount_", rowCount);
  run;
  %if (&_sddRowCount_ > 0) %then %do;
  /* create SAS datetime fields in the resulting dataset */
  data &sas_dsname(drop=createdOnSAS lastModifiedOnSAS);
      set &sas_dsname;
      label dateCreated="Date the object was created (SAS DateTime Format)" 
            dateLastModified="Date the object was last modified (SAS DateTime Format)" ;
      format dateCreated dateLastModified datetime. ;
   	  dateCreated=input(createdOnSAS,datetime.);
  	  dateLastModified=input(lastModifiedOnSAS,datetime.);
  run;
  /* sort the dataset by path and within path by fullpath */     
  data temp;
      set &sas_dsname;
      uppath=upcase(path) ;
      uppath=translate(uppath,'1','_');
      upname=upcase(fullpath);
      upname=translate(upname,'1','_');
  run;
  proc sort data=temp;
      by uppath upname;
  run;
  /* cleanup                                              */ 
  data &sas_dsname;
      set temp(drop=uppath upname);
  run;
  proc datasets library=work nolist;
      delete temp;
  run;
  %end;
  
%mend sasdrugdev_getversions ;
