%* sasdrugdev_updateacls                                                                                                    *;
%*                                                                                                                        *;
%* Sets permissions on SDD objects using a dataset as input.  The input dataset can be obtained by calling                *;
%* sasdrugdev_getacl.  Changes to permissions, additions of access control entries (ACE) and deletions of ACEs            *;
%* are done by adding or removing members from the dataset output of sasdrugdev_getacl.  The update is a full replacement *;
%* of the acls for each fullpath specified.                                                                               *;
%*                                                                                                                        *;
%* @macvar _sddRC_  SDD Return code                                                                                       *;
%* @macvar _sddMsg_ SDD Message                                                                                           *;
%*                                                                                                                        *;
%* @param sas_dsname = the input dsname that contains the complete list of acls desired for a container or object in SDD.     *;
%*                                                                                                                        *;

%macro sasdrugdev_updateacls(sas_dsname=%str());
	%global _sddRC_ _sddMsg_ ;
	%let _sddRC_=%str(-99);
	%let _sddMsg_=%str(The SAS Macro did not execute properly.  Unknown error.);

	* -;
	* Cleanup from any previous runs that may have left temp dataset behind.;
	* -;
	%let _tempACLUPDATE_tempDS_=%str(__SASMacro_acltemp__);

	%put NOTE: sasdrugdev_updateacls: deleting temporary dataset &_tempACLUPDATE_tempDS_ before processing;
	%if (%sysfunc(exist(&_tempACLUPDATE_tempDS_))>0) %then %do;
	   proc datasets library=work nolist;
	      delete &_tempACLUPDATE_tempDS_;
	   quit; run;
	%end;

	%local logMsgPrefix tempDsMsg operationFailed;
	%let logMsgPrefix=%str(ERROR: SAS Drug Development Macro: * );
	%let tempDsMsg=%str(Temporary sorted dataset work.&_tempACLUPDATE_tempDS_ will not be created.);

	* -;
	* Validate the input dataset
	* sets global macro variable _sddRC_=0 if validation successful 
	* -;
	%sasdrugdev_sdd_acls_ds(callingMacro=%str(sasdrugdev_updateacls), sas_dsn=&sas_dsname);
	%if (&_sddRC_ ne 0) %then %do;
		%put &logMsgPrefix SAS Drug Development macro sasdrugdev_updateacls;
		%put &logMsgPrefix Input dataset = &sas_dsname;
		%put &logMsgPrefix _sddRC_= &_sddRC_ ;
		%put &logMsgPrefix _sddMsg_= &_sddMsg_ ;
		%put &logMsgPrefix &tempDsMsg ;
		%put;
		%return;
	%end;

	* -;
	* If dataset is valid, process it.;
	* -;

	* setting a new macro variable for operation success because there are operations in updateACL   *;
	* that could set _sddRC_ to something other than 0 but still not be a failure.                   *;

	%put NOTE: sasdrugdev_updateacls: processing dataset &sas_dsname;

	* -;
	* Clean the dataset values
	* the dataset that the user sends in can have very corrupt data.
	* need to clean it up (without changing the values) as much as possible before processing.
	* Note: kstrip() of the required variables here did not help with sending them into the macro. 
	*       still had to do it there so I dropped it from here.;
	* -;
	data &_tempACLUPDATE_tempDS_;
		attrib fullpathforsort   length=$2048    label="Full Path Name in uppercase for sorting purposes";
		set &sas_dsname;
		* upcase values that should be case insensitive;
		* this is also for sorting purposes
		* NOTE: PrincipalName and all paths must be case-sensitive;
		aclType=upcase(aclType);
		principalType=upcase(principalType);
		* BUT fullpath needs to be upcased to insure sorting purposes for SAS but mixed case for SDD
		* so am creating a separate variable for sorting;
		fullpathforsort=upcase(fullpath);
	run;

	* -;
	* Sort the data
	* -;
	proc sort data=&_tempACLUPDATE_tempDS_    out=&_tempACLUPDATE_tempDS_;
		by fullpathforsort aclType;
	run;

	* -;
	* Process the records
	* -;
	data _null_;
		attrib recNumString            length=$8;
		attrib message    	           length=$200;
		attrib returnCode 	           length=8;
		attrib noExplicitPermsSetCode  length=8;
		attrib noExplicitPermsMsg      length=$200;
		attrib permsNotSetCount        length=8; 
		attrib statusPrefix  	       length=$40;
		attrib logMsgPrefix  	       length=$40;
		attrib msgPrefix               length=$40;

		retain noExplicitPermsSetCode permsNotSetCount;

		set &_tempACLUPDATE_tempDS_ end=eof;
		by fullpathforsort aclType;

		* only instantiate once *;
		if  (_n_ = 1) then do;
			declare javaobj results("com/sas/hls/macro/results/DefaultResult"); 
			results.callIntMethod("getCode", "NO_EXPLICITPERMS_CODE", noExplicitPermsSetCode);
			declare javaobj cmd("com/sas/hls/macro/command/ACLUpdateCommand", results);
			link processResult;
			cmd.callVoidMethod("setDatasetName", "%bquote(&_tempACLUPDATE_tempDS_)");
		end;

		recNumString=strip(put(_n_, best.));
		cmd.callVoidMethod("setRecordNumber", recNumString);
		failure=0;

		if (first.aclType) then do;
			permsNotSetCount=0;
			* validate the path and initiate the resource ;
			cmd.callVoidMethod("initializeACL", results, kstrip(fullpath), kstrip(aclType));
			link processResult;
		end;

		* get the acl entry to update or create a new one ;
		cmd.callVoidMethod("setEntryToUpdate", results, kstrip(principalType), kstrip(principalName), kstrip(grpSrcCtxt));
		link processResult;

		* set the permissions on the acl entry ;
 		cmd.callVoidMethod("setPermissions", results, strip(adminPermission), strip(readPermission), strip(writePropPerm), strip(writeContentPerm), strip(deletePermission));
		link processResult;

		if (last.aclType) then do;
			* finally, update the acl entry ;
			cmd.callVoidMethod("update", results);
			link processResult;
			if NOT(eof) then do;
				* the only way to be here is if there was no failure;
				* do not want to do this if eof because it will fall through for that;
				link printMessages;
			end;
		end;

		if (eof) then do;
			* the only way to be here is if eof and nothing failed;
			link printMessages;
		end;

		return;

		** check the results of calls so far. ;
		processResult:
			results.callIntMethod("getReturnCode", returnCode);
			results.callStringMethod("getReturnMessage", message);
			if (returnCode eq 0) then
				return;
			if (returnCode eq noExplicitPermsSetCode) then do;
				* can only be here from setPermissions;
				permsNotSetCount+1;
				return;
			end;
			* can only be here if there was a failure;
			failure=1;
			link printMessages;
		   	STOP;
		return;

		** print messages to the log if (1) an error stopped the processing or (2) updated a complete ACL. ;
		printMessages:
			* set the log messages;
			if (failure) then do;
				statusPrefix = "ERROR";
				end;
			else if (permsNotSetCount gt 0) then do;
				statusPrefix = "WARNING";
				end;
			else do;
				statusPrefix = "NOTE";
				end;

			logMsgPrefix=compress(statusPrefix) || ":";
			msgPrefix=compress(statusPrefix) || "- ";

			* print the messages;
			put logMsgPrefix "SAS Drug Development Macro: *  SAS Drug Development macro sasdrugdev_updateacls";
			put logMsgPrefix "SAS Drug Development Macro: *  Input dataset = &sas_dsname";
			put logMsgPrefix "SAS Drug Development Macro: *  Processing ACL: " aclType "for path " fullpath; 
			put logMsgPrefix "SAS Drug Development Macro: *  _sddRC_= " returnCode ;
			if (returnCode eq 0) then do;
				put logMsgPrefix "SAS Drug Development Macro: *  _sddMsg_= " message ;
			end;
			else do;
				put logMsgPrefix "SAS Drug Development Macro: *  _sddMsg_= " ;
				put msgPrefix "-"  message ;
			end;

			if (permsNotSetCount gt 0) then do;
				noExplicitPermsMsg = msgPrefix || " " || strip(put(permsNotSetCount,best.)) ||
									" records had no permissions explicitly set for the ACL being processed. " ||
                                    "The ACE for those records was not added to or was removed from the ACL.";
				put noExplicitPermsMsg;
			end;
			put;

			* capture values for post-processing;
			call symputx("_sddRC_", returnCode);
			call symputx("_sddMsg_", message);
			call symputx("operationFailed", failure);

		return;

	run;

	%if (&operationFailed eq 0) %then %do;
		%if (%sysfunc(exist(&_tempACLUPDATE_tempDS_))>0) %then %do;
			%put NOTE: sasdrugdev_updateacls: deleting temporary dataset &_tempACLUPDATE_tempDS_;
		   	proc datasets library=work nolist;
				delete &_tempACLUPDATE_tempDS_;
		   	quit; run;
		%end;
	%end;
	%else %do;
		%put ERROR: Temporary sorted dataset work.&_tempACLUPDATE_tempDS_ is not being deleted.;
	%end;

%mend sasdrugdev_updateacls;
